# Copyright (C) 2016 Catalyst IT Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from adjutantclient.common import base

from six.moves.urllib import parse


class Token(base.Resource):
    pass


class TokenParam(base.Resource):
    pass


class TokenManager(base.BaseManager):
    resource_class = Token

    def get(self, token_id):
        """Get details on a particular token object"""
        url = 'tokens/%s' % token_id
        return self._get(url)

    def list(self, **kwargs):
        """Get a list of tokens.

        :rtype: list of :class:`Token`
        """
        url = '/tokens?%(params)s' % {
            'params': parse.urlencode(kwargs, True)
        }
        return self._list(url, 'tokens')

    def submit(self, token_id, parameters):
        url = 'tokens/%s' % token_id
        return self._post(url, parameters)

    def reissue(self, task_id):
        """Given a task id, reissues the tokens associated with that task"""
        url = 'tokens'
        data = {
            'task': task_id
        }
        return self._post(url, data)

    def clear_expired(self):
        """Clear all expired tokens."""
        url = '/tokens?'
        return self._delete(url)



# Copyright (C) 2016 Catalyst IT Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from six.moves.urllib import parse

from adjutantclient.common import base


class QuotaManager(base.BaseManager):

    def get(self, regions=None):
        """Gets data about current quota settings"""
        url = '/openstack/quotas/'
        if regions:
            url += '?%s' % parse.urlencode({
                'regions': regions
            })
        return self.client.get(url).json()

    def update(self, size, regions=None):
        """Updates the quota to a specified size.

        If region is not set it will update all regions
        """
        url = '/openstack/quotas/'
        fields = {
            'size': size,
            'regions': regions
        }
        return self.client.post(url, data=fields)



# Copyright (C) 2016 Catalyst IT Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from adjutantclient.common import base


class SignupManager(base.BaseManager):

    def post(self, username, email, project_name, setup_network):
        url = '/openstack/sign-up'
        fields = {
            'username': username,
            'email': email,
            'project_name': project_name,
            'setup_network': setup_network
        }
        return self.client.post(url, data=fields)



# Copyright (C) 2016 Catalyst IT Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from six.moves.urllib import parse

from adjutantclient.common import base


class Notification(base.Resource):
    pass


class NotificationManager(base.ManagerWithFind):
    resource_class = Notification

    def get(self, note_id):
        return self._get("/notifications/%s" % base.getid(note_id))

    def list(self, **kwargs):
        """Get a list of notifications.

        :rtype: list of :class:`Notification`
        """
        url = '/notifications?%(params)s' % {
            'params': parse.urlencode(kwargs, True)
        }
        return self._list(url, 'notifications')

    def acknowledge(self, note_id=None, note_list=[]):
        """Acknowledge a single notification or many."""
        if note_id:
            data = {'acknowledged': True}
            url = 'notifications/%s' % note_id
            return self._post(url, data)
        elif note_list:
            data = {'notifications': note_list}
            return self._post('notifications', data)



# Copyright (c) 2016 Catalyst IT Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import logging

from osc_lib.command import command
from osc_lib.i18n import _

from adjutantclient import client as adjutant_client

LOG = logging.getLogger(__name__)


def _list_tokens(client, filters={}):
    tokens = client.tokens.list(filters=filters)

    headers = ['Token', 'Task', 'Expires on', 'Created on']
    rows = [[token.token, token.task, token.expires,
             token.created_on] for token in tokens]

    return headers, rows


class TokenList(command.Lister):
    """Lists adjutant tokens. """

    def get_parser(self, prog_name):
        parser = super(TokenList, self).get_parser(prog_name)

        parser.add_argument(
            '--filters', metavar='<filters>',
            required=False,
            help=_('JSON containing filters for tokens.'),
            default={})

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.admin_logic
        return _list_tokens(client, parsed_args.filters)


class TokenShow(command.ShowOne):
    """Show details of one token."""

    def get_parser(self, prog_name):
        parser = super(TokenShow, self).get_parser(prog_name)

        parser.add_argument(
            'token', metavar='<token_id>',
            help=_("The token."))
        parser.add_argument(
            '--bypass-url', metavar='<bypass-url>', default=None,
            help=_('Bypasss URL for unauthenticated access to the endpoint.'))
        return parser

    def take_action(self, parsed_args):
        if not parsed_args.bypass_url:
            self.app.client_manager._auth_required = True
            self.app.client_manager.setup_auth()
            client = self.app.client_manager.admin_logic
        else:
            client = adjutant_client.Client("1", parsed_args.bypass_url)
        token = client.tokens.get(parsed_args.token)
        return zip(*(token.to_dict()).items())


class TokenSubmit(command.Command):
    """Submit token data."""
    def get_parser(self, prog_name):
        parser = super(TokenSubmit, self).get_parser(prog_name)

        parser.add_argument(
            'token', metavar='<token_id>', help=_('The token.'))
        parser.add_argument(
            'data', metavar='<token_data>',
            help=_('Submission data for the token. Must be valid json.'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.admin_logic
        resp = client.tokens.submit(
            parsed_args.token, json.loads(parsed_args.data))
        print('Success', ' '.join(resp.notes))


class TokenClear(command.Lister):
    """Clear Expired tokens, admin only."""

    def take_action(self, parsed_args):
        client = self.app.client_manager.admin_logic
        resp = client.tokens.clear_expired()
        print('Success. ' + ' '.join(resp.json()['notes']))
        return _list_tokens(client)



# Copyright (c) 2016 Catalyst IT Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from osc_lib.command import command
from osc_lib.i18n import _
from osc_lib import utils

from adjutantclient import client as adjutant_client
from adjutantclient import exc

LOG = logging.getLogger(__name__)


class UserList(command.Lister):
    """Lists users in the currently scoped project. """

    def take_action(self, parsed_args):
        client = self.app.client_manager.admin_logic
        project_users = client.users.list()
        headers = [
            'id', 'name', 'email', 'roles', 'cohort', 'status']

        rows = [[user.id, user.name, user.email,
                 user.roles, user.cohort, user.status]
                for user in project_users]

        return headers, rows


class UserShow(command.ShowOne):
    """Show details of one user."""

    def get_parser(self, prog_name):
        parser = super(UserShow, self).get_parser(prog_name)

        parser.add_argument(
            'user', metavar='<user>',
            help=_("The user's ID or name."))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.admin_logic
        # This ends up for names doing multiple requests, it may
        # be better to do something slightly different here
        user_id = utils.find_resource(client.users, parsed_args.user)
        user = client.users.get(user_id)
        return zip(*(user.to_dict()).items())


class UserInvite(command.Command):
    """Invites a user to become a member of a project.

    User does not need to have an existing openstack account.
    """

    def get_parser(self, prog_name):
        parser = super(UserInvite, self).get_parser(prog_name)

        parser.add_argument(
            '--username', metavar='<username>',
            default=None,
            help=_('The username for the new user.'))
        parser.add_argument(
            'email', metavar='<email>',
            help=_('Email address of user to invite'))
        parser.add_argument(
            'roles', metavar='<role>', nargs='+',
            help=_('Roles to give to the user.'))
        return parser

    def take_action(self, parsed_args):
        if not parsed_args.roles:
            parsed_args.roles = ['Member']
        client = self.app.client_manager.admin_logic
        client.users.invite(
            username=parsed_args.username, email=parsed_args.email,
            role_list=parsed_args.roles)
        print("User invited")


class UserInviteCancel(command.Command):
    """Cancel invite(s) to a project."""
    def get_parser(self, prog_name):
        parser = super(UserInviteCancel, self).get_parser(prog_name)

        parser.add_argument(
            'user', metavar='<user>',
            nargs='+',
            help=_("The user's name or id."))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.admin_logic
        for user in parsed_args.user:
            try:
                user_id = client.users.find(name=user).id
            except exc.NotFound:
                user_id = client.users.find(id=user).id
            client.users.cancel(user_id=user_id)
        print("Invite(s) Cancelled")


class UserRoleAdd(command.Command):
    """Add a role to a user."""
    def get_parser(self, prog_name):
        parser = super(UserRoleAdd, self).get_parser(prog_name)

        parser.add_argument(
            'user', metavar='<user>',
            help=_("The user's name or id.."))
        parser.add_argument(
            'role', metavar='<role>',
            help=_("The role's name or id."))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.admin_logic

        role = utils.find_resource(client.managed_roles, parsed_args.role)
        user = utils.find_resource(client.users, parsed_args.user)
        if client.user_roles.add(user.id, role=role.name):
            print(_("Role added"))


class UserRoleRemove(command.Command):
    """Remove a role from a user."""
    def get_parser(self, prog_name):
        parser = super(UserRoleRemove, self).get_parser(prog_name)

        parser.add_argument(
            'user', metavar='<user>',
            help=_("The user's name or id.."))
        parser.add_argument(
            'role', metavar='<user>',
            help=_("The role's name or id."))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.admin_logic
        role = utils.find_resource(client.managed_roles, parsed_args.role)
        user = utils.find_resource(client.users, parsed_args.user)

        if client.user_roles.remove(user.id, role=role.name):
            print(_("Role removed"))


class UserRoleList(command.Lister):
    """Lists the roles a user has on a project"""
    def get_parser(self, prog_name):
        parser = super(UserRoleList, self).get_parser(prog_name)

        parser.add_argument(
            'user', metavar='<user>',
            help=_("Name or ID of user."))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.admin_logic

        user = utils.find_resource(client.users, parsed_args.user)
        kwargs = {'user': user.id}
        roles = [[role.id, role.name] for role
                 in client.user_roles.list(**kwargs)]
        return ['id', 'name'], roles


class ManageableRolesList(command.Lister):
    """Lists roles able to be managed by the current user """
    def take_action(self, parsed_args):
        client = self.app.client_manager.admin_logic
        roles = client.managed_roles.list()

        headers = ['id', 'name']
        rows = [[role.id, role.name] for role in roles]
        return headers, rows


class PasswordReset(command.Command):
    """Force password reset for a user, admin only. """

    def get_parser(self, prog_name):
        parser = super(PasswordReset, self).get_parser(prog_name)

        parser.add_argument(
            'email', metavar='<email>',
            help=_("Email address of the user."))
        parser.add_argument(
            '--username', metavar='<username>', default=None,
            help=_('Username of the account to reset if the username '
                   'is different than the email'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.admin_logic

        data = {'email': parsed_args.email}
        if parsed_args.username:
            data['username'] = parsed_args.username

        client.users.password_force_reset(data)
        print("Task has been sucessfully submitted.")
        print("If a user with that email exists, a reset "
              "token will be issued.")


class PasswordForgot(command.Command):
    """Links to user forgotten password endpoint, does not require auth."""
    auth_required = False

    def get_parser(self, prog_name):
        parser = super(PasswordForgot, self).get_parser(prog_name)

        parser.add_argument(
            'email', metavar='<email>',
            help=_("Email address of the user."))
        parser.add_argument(
            '--username', metavar='<username>', default=None,
            help=_('Username of the account to reset if the username '
                   'is different than the email'))
        parser.add_argument(
            '--bypass-url', metavar='<bypass-url>', default=None,
            help=_('Bypasss URL for unauthenticated access to the endpoint.'))
        return parser

    def take_action(self, parsed_args):
        if not parsed_args.bypass_url:
            self.app.client_manager._auth_required = True
            self.app.client_manager.setup_auth()
            client = self.app.client_manager.admin_logic
        else:
            client = adjutant_client.Client(1, parsed_args.bypass_url)

        client.users.password_forgot(parsed_args.email, parsed_args.username)
        print("Task has been sucessfully submitted.")
        print("If a user with that email exists, a reset "
              "token will be issued.")



# Copyright (C) 2016 Catalyst IT Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from six.moves.urllib import parse

from adjutantclient.common import base


class Task(base.Resource):
    pass


class TaskManager(base.ManagerWithFind):
    resource_class = Task

    def get(self, task_id):
        return self._get("/tasks/%s" % base.getid(task_id))

    def list(self, **kwargs):
        """Get a list of tasks.

        :rtype: list of :class:`Task`
        """
        url = '/tasks?%(params)s' % {
            'params': parse.urlencode(kwargs, True)
        }
        return self._list(url, 'tasks')

    def update(self, task_id, data):
        """Update a task with new data and rerun pre-approve validation."""
        url = 'tasks/%s' % task_id
        return self._put(url, data)

    def approve(self, task_id):
        """Approve a task.

        If already approved will rerun post-approve validation
        and reissue/resend token.
        """
        data = {'approved': True}
        url = 'tasks/%s' % task_id
        return self._post(url, data)

    def cancel(self, task_id):
        """Cancel a task."""
        url = 'tasks/%s' % task_id
        return self._delete(url)



# Copyright (C) 2016 Catalyst IT Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from adjutantclient.common import http
from adjutantclient.v1 import notifications
from adjutantclient.v1 import quota
from adjutantclient.v1 import roles
from adjutantclient.v1 import signup
from adjutantclient.v1 import status
from adjutantclient.v1 import tasks
from adjutantclient.v1 import tokens
from adjutantclient.v1 import users


class Client(object):
    """Client for the Adjutant v1 API.

    :param string endpoint: A user-supplied endpoint URL for the adjutant
                            service.
    :param string token: Token for authentication.
    :param integer timeout: Allows customization of the timeout for client
                            http requests. (optional)
    """

    def __init__(self, *args, **kwargs):
        """Initialize a new client for the Adjutant v1 API."""
        self.http_client = http._construct_http_client(*args, **kwargs)

        self.users = users.UserManager(self.http_client)
        self.user_roles = roles.UserRoleManager(self.http_client)
        self.managed_roles = roles.ManagedRoleManager(self.http_client)
        self.tokens = tokens.TokenManager(self.http_client)
        self.tasks = tasks.TaskManager(self.http_client)
        self.signup = signup.SignupManager(self.http_client)
        self.notifications = notifications.NotificationManager(
            self.http_client)
        self.status = status.StatusManager(self.http_client)
        self.quota = quota.QuotaManager(self.http_client)



# Copyright (c) 2016 Catalyst IT Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import logging

from osc_lib.command import command
from osc_lib.i18n import _


LOG = logging.getLogger(__name__)


def _show_notification(notification_id, client, formatter):
    notification = client.notifications.get(notification_id)
    if formatter == 'table':
        notification._info['notes'] = json.dumps(
            notification.notes, indent=2)
    return zip(*(notification.to_dict()).items())


class NotificationList(command.Lister):
    """Lists adjutant notifications. """

    def get_parser(self, prog_name):
        parser = super(NotificationList, self).get_parser(prog_name)

        parser.add_argument(
            '--filters', metavar='<filters>',
            required=False,
            help=_('JSON containing filters for the notifications.'),
            default={})

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.admin_logic

        kwargs = {}
        if parsed_args.filters:
            kwargs['filters'] = parsed_args.filters

        notifications = client.notifications.list(filters=parsed_args.filters)

        headers = ['uuid', 'task', 'acknowledged', 'created_on']

        rows = [[notif.uuid, notif.task, notif.acknowledged,
                 notif.created_on] for notif in notifications]

        return headers, rows


class NotificationShow(command.ShowOne):
    """Show details of one notification."""

    def get_parser(self, prog_name):
        parser = super(NotificationShow, self).get_parser(prog_name)

        parser.add_argument(
            'notification_id', metavar='<notification_id>',
            help=_("The notification ID."))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.admin_logic
        return _show_notification(parsed_args.notification_id, client,
                                  parsed_args.formatter)


class NotificationAcknowledge(command.Command):
    """Approve and show details of a notification."""
    def get_parser(self, prog_name):
        parser = super(NotificationAcknowledge, self).get_parser(prog_name)

        parser.add_argument('note_ids', metavar='<noteids>', nargs='+',
                            help=_('Notification IDs to acknowledge.'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.admin_logic
        resp = client.notifications.acknowledge(note_list=parsed_args.note_ids)
        print('Success', ' '.join(resp.notes))



# Copyright (c) 2016 Catalyst IT Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
import six

from osc_lib.command import command
from osc_lib.i18n import _


LOG = logging.getLogger(__name__)


class QuotaShow(command.Lister):
    """Displays current quota information.

    If not given a region it will print basic details of the state of
    the quotas. If given a region it will print all details for it.
    """

    def get_parser(self, prog_name):
        parser = super(QuotaShow, self).get_parser(prog_name)

        parser.add_argument(
            '--region', metavar='<region>', required=False,
            help=_("Single region to display details on."))

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.admin_logic

        if not parsed_args.region:
            quota_data = client.quota.get()

            headers = [
                'Region', 'Current Size', 'Preapproved Sizes']

            rows = []
            for region in quota_data['regions']:
                rows.append([
                    region['region'],
                    region['current_quota_size'],
                    ", ".join(region['quota_change_options']),
                ])
            return headers, rows
        else:
            quota_data = client.quota.get(regions=parsed_args.region)
            headers = ['Service', 'Resource', 'Current Quota', 'Current Usage']
            region = quota_data['regions'][0]

            rows = []
            for service, service_detail in six.iteritems(
                    region['current_usage']):
                for resource, value in six.iteritems(service_detail):
                    current_quota = region['current_quota'][service].get(
                        resource)
                    rows.append([service, resource, current_quota, value])
            return headers, rows


class QuotaSizes(command.Lister):
    """Displays possible quota sizes."""

    def take_action(self, parsed_args):
        client = self.app.client_manager.admin_logic

        quota_data = client.quota.get()

        headers = [
            'Size Name', 'Service', 'Resource', 'Value']

        rows = []
        for size, size_details in six.iteritems(quota_data['quota_sizes']):
            for service, service_details in six.iteritems(size_details):
                for resource, value in six.iteritems(service_details):
                    rows.append([size, service, resource, value])

        return headers, rows


class QuotaTasks(command.Lister):
    """Displays quota tasks."""

    def take_action(self, parsed_args):
        client = self.app.client_manager.admin_logic

        quota_data = client.quota.get()

        headers = [
            'ID', 'Regions', 'Proposed Size', 'Requested By', 'Created On',
            'valid', 'status',
        ]

        rows = []
        for task in quota_data['active_quota_tasks']:
            rows.append([
                task['id'],
                ", ".join(task['regions']),
                task['size'],
                task['request_user'],
                task['task_created'],
                task['valid'],
                task['status'],
            ])

        return headers, rows


class QuotaUpdate(command.Command):
    """Submits a quota update task."""

    def get_parser(self, prog_name):
        parser = super(QuotaUpdate, self).get_parser(prog_name)

        parser.add_argument(
            'size', metavar='<size>',
            help=_("The size to update to."))
        parser.add_argument(
            '--regions', metavar='<regions>', nargs='+', required=False,
            help=_("Regions to update the quota on."))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.admin_logic
        if parsed_args.regions:
            status = client.quota.update(parsed_args.size, parsed_args.regions)
        else:
            # NOTE(amelia): regions aren't set the API will update all of them
            status = client.quota.update(parsed_args.size)
        status_code = status.status_code

        if status_code == 200:
            if parsed_args.regions:
                print("Regions: %s quota updated to size %s."
                      % (parsed_args.regions, parsed_args.size))
            else:
                print("All regions' quota updated to size %s."
                      % (parsed_args.size))
        elif status_code == 202:
            print("The task has been created however requires admin approval"
                  " before executing.")



# Copyright (c) 2016 Catalyst IT Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import logging
from osc_lib.command import command
from osc_lib.i18n import _


LOG = logging.getLogger(__name__)


def _show_task(task_id, client, formatter):
    task = client.tasks.get(task_id)
    if formatter == 'table':
        task._info['actions'] = json.dumps(
            getattr(task, 'actions', ""), indent=2)
        task._info['keystone_user'] = json.dumps(
            getattr(task, 'keystone_user', ""), indent=2)
        task._info['action_notes'] = json.dumps(
            getattr(task, 'action_notes', ""), indent=2)
        task._info['approved_by'] = json.dumps(
            getattr(task, 'approved_by', ""), indent=2)
    return zip(*(task.to_dict()).items())


class TaskList(command.Lister):
    """Lists adjutant tasks. """

    def get_parser(self, prog_name):
        parser = super(TaskList, self).get_parser(prog_name)

        parser.add_argument(
            '--limit', metavar='<limit>',
            required=False,
            help=_('Limit the number of task responses.'))
        parser.add_argument(
            '--filters', metavar='<filters>',
            required=False,
            help=_('JSON containing filters for the tasks.'),
            default={})

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.admin_logic

        kwargs = {'filters': parsed_args.filters}

        if parsed_args.limit:
            kwargs['tasks_per_page'] = parsed_args.limit

        tasks = client.tasks.list(**kwargs)

        headers = [
            'UUID', 'Task Type', 'Created on',
            'Approved on', 'Completed on', 'Cancelled']

        rows = [[task.uuid, task.task_type, task.created_on,
                 task.approved_on, task.completed_on, task.cancelled]
                for task in tasks]

        return headers, rows


class TaskShow(command.ShowOne):
    """Show details of one task."""

    def get_parser(self, prog_name):
        parser = super(TaskShow, self).get_parser(prog_name)

        parser.add_argument(
            'task_id', metavar='<taskid>',
            help=_("The task ID."))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.admin_logic
        return _show_task(parsed_args.task_id, client, parsed_args.formatter)


class TaskApprove(command.ShowOne):
    """Approve and show details of a task."""
    def get_parser(self, prog_name):
        parser = super(TaskApprove, self).get_parser(prog_name)

        parser.add_argument(
            'task_id', metavar='<taskid>',
            help=_("The task ID."))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.admin_logic
        client.tasks.approve(parsed_args.task_id)
        return _show_task(parsed_args.task_id, client, parsed_args.formatter)


class TaskCancel(command.ShowOne):
    """Approve and show details of a task."""
    def get_parser(self, prog_name):
        parser = super(TaskCancel, self).get_parser(prog_name)

        parser.add_argument(
            'task_id', metavar='<taskid>',
            help=_("The task ID."))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.admin_logic
        client.tasks.cancel(task_id=parsed_args.task_id)
        return _show_task(parsed_args.task_id, client, parsed_args.formatter)


class TaskUpdate(command.ShowOne):
    """Update data and re-run pre-approve for a task."""
    def get_parser(self, prog_name):
        parser = super(TaskUpdate, self).get_parser(prog_name)

        parser.add_argument(
            'task_id', metavar='<taskid>',
            help=_("The task ID."))
        parser.add_argument(
            'data', metavar='<data>',
            help=_("New JSON action data."))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.admin_logic
        client.tasks.update(task_id=parsed_args.task_id,
                            data=json.loads(parsed_args.data))
        return _show_task(parsed_args.task_id, client, parsed_args.formatter)


class TaskTokenReissue(command.Command):
    """Reissues a token for the provided pending task """
    def get_parser(self, prog_name):
        parser = super(TaskTokenReissue, self).get_parser(prog_name)

        parser.add_argument('task_id', metavar='<task_id>',
                            help=_('The task ID.'))
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.admin_logic
        resp = client.tokens.reissue(task_id=parsed_args.task_id)
        print('Success', ' '.join(resp.notes))



# Copyright (c) 2016 Catalyst IT Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import logging

from osc_lib.command import command
from osc_lib.i18n import _

from adjutantclient import client as adjutant_client


LOG = logging.getLogger(__name__)


class Signup(command.Command):
    """Submits a sign-up request."""
    auth_required = False

    def get_parser(self, prog_name):
        parser = super(Signup, self).get_parser(prog_name)

        parser.add_argument(
            '--user', metavar='<user>',
            required=False,
            help=_('Username for new account. '
                   'May not be required depending on API configuration.'),
            default=None)
        parser.add_argument(
            'email', metavar='<email>',
            help=_('New account email address.'))
        parser.add_argument(
            'project_name', metavar='<project_name>',
            help=_('Name of the new project'))
        parser.add_argument(
            '--bypass-url', metavar='<bypass-url>', default=None,
            help=_('Bypasss URL for unauthenticated access to the endpoint.'))
        parser.add_argument(
            '--setup-network', action='store_true',
            help=_('Create a default network during project creation.'))

        return parser

    def take_action(self, parsed_args):
        if not parsed_args.bypass_url:
            # NOTE(amelia): Assumes that this is an already authenticated
            # user wanting to access and submit a sign up (I.E. an admin)
            self.app.client_manager._auth_required = True
            self.app.client_manager.setup_auth()
            client = self.app.client_manager.admin_logic
        else:
            client = adjutant_client.Client(1, parsed_args.bypass_url)

        signup = client.signup.post(parsed_args.user, parsed_args.email,
                                    parsed_args.project_name,
                                    parsed_args.setup_network).json()
        print(json.dumps(signup, indent=2))



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging
import os

from keystoneauth1 import adapter
from keystoneauth1.identity import v3
from keystoneauth1 import session as ks_session
from oslo_utils import importutils
import requests

from adjutantclient._i18n import _
from adjutantclient import exc

LOG = logging.getLogger(__name__)
USER_AGENT = 'python-adjutantclient'
CHUNKSIZE = 1024 * 64  # 64kB
SENSITIVE_HEADERS = ('X-Auth-Token',)
osprofiler_web = importutils.try_import("osprofiler.web")


def get_response_body(resp):
    body = resp.content
    if 'application/json' in resp.headers.get('content-type', ''):
        try:
            body = resp.json()
        except ValueError:
            LOG.error('Could not decode response body as JSON')
    else:
        body = None
    return body


def get_system_ca_file():
    """Return path to system default CA file."""
    # Standard CA file locations for Debian/Ubuntu, RedHat/Fedora,
    # Suse, FreeBSD/OpenBSD, MacOSX, and the bundled ca
    ca_path = ['/etc/ssl/certs/ca-certificates.crt',
               '/etc/pki/tls/certs/ca-bundle.crt',
               '/etc/ssl/ca-bundle.pem',
               '/etc/ssl/cert.pem',
               '/System/Library/OpenSSL/certs/cacert.pem',
               requests.certs.where()]
    for ca in ca_path:
        LOG.debug("Looking for ca file %s", ca)
        if os.path.exists(ca):
            LOG.debug("Using ca file %s", ca)
            return ca
    LOG.warning("System ca file could not be found.")


class SessionClient(adapter.LegacyJsonAdapter):
    """HTTP client based on Keystone client session."""

    def request(self, url, method, **kwargs):
        redirect = kwargs.get('redirect')
        kwargs.setdefault('user_agent', USER_AGENT)

        if 'data' in kwargs:
            kwargs['json'] = kwargs.pop('data')

        resp, body = super(SessionClient, self).request(
            url, method,
            raise_exc=False,
            **kwargs)

        if 400 <= resp.status_code < 600:
            raise exc.from_response(resp)
        elif resp.status_code in (301, 302, 305):
            if redirect:
                location = resp.headers.get('location')
                path = self.strip_endpoint(location)
                resp = self.request(path, method, **kwargs)
        elif resp.status_code == 300:
            raise exc.from_response(resp)

        return resp

    def credentials_headers(self):
        return {}

    def strip_endpoint(self, location):
        if location is None:
            message = _("Location not returned with 302")
            raise exc.InvalidEndpoint(message=message)
        if (self.endpoint_override is not None and
                location.lower().startswith(self.endpoint_override.lower())):
            return location[len(self.endpoint_override):]
        else:
            return location


def _construct_http_client(endpoint=None, username=None, password=None,
                           include_pass=None, endpoint_type=None,
                           auth_url=None, **kwargs):
    session = kwargs.pop('session', None)
    auth = kwargs.pop('auth', None)

    if not session:
        if not auth and password:
            auth = v3.Password(auth_url=auth_url,
                               username=username,
                               password=password,
                               **kwargs)
            kwargs.pop('project_id', None)
            kwargs.pop('project_name', None)
            kwargs.pop('domain_id', None)
            kwargs.pop('domain_name', None)
            kwargs.pop('project_domain_id', None)
            kwargs.pop('project_domain_name', None)
            kwargs.pop('user_domain_id', None)
            kwargs.pop('user_domain_name', None)

        ca_path = kwargs.get('ca_file') or get_system_ca_file()
        session = ks_session.Session(auth=auth,
                                     verify=ca_path)

    if 'endpoint_override' not in kwargs and endpoint:
        kwargs['endpoint_override'] = endpoint

    if 'service_type' not in kwargs:
        kwargs['service_type'] = 'admin-logic'

    if 'interface' not in kwargs and endpoint_type:
        kwargs['interface'] = endpoint_type

    return SessionClient(session, auth=auth, **kwargs)



# Copyright (C) 2016 Catalyst IT Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from six.moves.urllib import parse

from adjutantclient.common import base


class User(base.Resource):
    def __repr__(self):
        return "<User %s>" % self._info

    def invite(self, **fields):
        return self.manager.create(self.identifier, **fields)

    def get(self):
        # set_loaded() first ... so if we have to bail, we know we tried.
        self._loaded = True
        if not hasattr(self.manager, 'get'):
            return

        new = self.manager.get(self.identifier)
        if new:
            self._add_details(new._info)

    @property
    def identifier(self):
        return '%s/%s' % (self.stack_name, self.id)


class UserManager(base.ManagerWithFind):
    resource_class = User

    def get(self, user):
        return self._get("/openstack/users/%s" % base.getid(user))

    def list(self, **kwargs):
        """Get a list of users.

        :param limit: maximum number of users to return
        :param marker: begin returning users that appear later in the user
                       list than that represented by this user id
        :rtype: list of :class:`User`
        """
        def paginate(params):
            '''Paginate users, even if more than API limit.'''
            current_limit = int(params.get('limit') or 0)
            url = 'openstack/users?%s' % parse.urlencode(params, True)
            users = self._list(url, 'users')
            for user in users:
                yield user

            num_users = len(users)
            remaining_limit = current_limit - num_users
            if remaining_limit > 0 and num_users > 0:
                params['limit'] = remaining_limit
                params['marker'] = user.id
                for stack in paginate(params):
                    yield stack

        params = {}
        # if 'filters' in kwargs:
        #     filters = kwargs.pop('filters')
        #     params.update(filters)

        for key, value in (kwargs).items():
            if value:
                params[key] = value

        return paginate(params)

    def invite(self, username, email, role_list):
        """Invite a user to the current project."""

        fields = {
            'username': username,
            'email': email,
            'roles': role_list
        }
        return self.client.post('openstack/users', data=fields)

    def cancel(self, user_id):
        """Cancel a user invite task."""
        url = 'openstack/users/%s' % user_id
        return self._delete(url)

    def revoke(self, user_id):
        """Revoke all user roles on project."""
        # NOTE(adriant): This doesn't work yet.
        # Uses the same endpoint as cancel, but doesn't
        # yet actually revoke all roles if an actual user
        # rather than an invite.
        return self.cancel(user_id)

    def password_forgot(self, email, username=None):
        """Forgot password email submission."""
        params = {"email": email}
        if username:
            params['username'] = username

        return self.client.post("openstack/users/password-reset",
                                data=params)

    def password_force_reset(self, email):
        """Force a password reset for a user.

        This is an admin only function.
        """
        params = {"email": email}
        return self.client.post(
            "openstack/users/password-set", data=params)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_utils import importutils


def Client(version, *args, **kwargs):
    module = importutils.import_versioned_module(
        'adjutantclient', version, 'client')
    client_class = getattr(module, 'Client')
    return client_class(*args, **kwargs)



# Copyright 2014 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from osc_lib import utils

LOG = logging.getLogger(__name__)

DEFAULT_OS_ADMIN_LOGIC_VERSION = '1'
DEFAULT_API_VERSION = '1'
API_VERSION_OPTION = 'os_admin_logic_version'
API_NAME = "admin_logic"
API_VERSIONS = {
    "1": "adjutantclient.v1.client.Client",
}


def make_client(instance):
    """Returns an adjutant service client."""
    version = instance._api_version[API_NAME]
    try:
        version = int(version)
    except ValueError:
        version = float(version)

    version = 1

    adjutant_client = utils.get_client_class(
        API_NAME,
        version,
        API_VERSIONS)

    LOG.debug('Instantiating adjutant client: %s', adjutant_client)

    kwargs = {'region_name': instance.region_name}

    if instance.session:
        kwargs.update({'session': instance.session})
    else:
        kwargs.update({'auth_url': instance.auth.auth_url,
                       'username': instance.auth_ref.username,
                       'token': instance.auth_ref.auth_token})

    client = adjutant_client(**kwargs)

    return client


def build_option_parser(parser):
    """Hook to add global options."""
    parser.add_argument(
        '--os-admin-logic-version',
        metavar='<admin-logic-version>',
        default=utils.env(
            'OS_ADMIN_LOGIC_VERSION',
            default=DEFAULT_OS_ADMIN_LOGIC_VERSION),
        help=('Client version, default=' +
              DEFAULT_OS_ADMIN_LOGIC_VERSION +
              ' (Env: DEFAULT_OS_ADMIN_LOGIC_VERSION)'))
    return parser



# Copyright (C) 2016 Catalyst IT Ltd
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from six.moves.urllib import parse

from adjutantclient.common import base
from adjutantclient import exc


class Role(base.Resource):
    pass


class ManagableRole(base.Resource):
    pass


class ManagedRoleManager(base.ManagerWithFind):
    resource_class = ManagableRole

    def list(self, **kwargs):
        """Get a list of roles that can be managed.

        :rtype: list of :class:`Role`
        """
        params = {}
        url = '/openstack/roles?%(params)s' % {
            'params': parse.urlencode(params, True)
        }
        return self._list(url, 'roles')

    def get(self, role_id):
        """Get a role by role_id"""
        # Right now the only way is to list them all, then iterate.
        # Perhaps a filter or new endpoint would be useful here.
        roles = self.list()
        for role in roles:
            if role.id == role_id:
                return role
        raise exc.HTTPNotFound()


class UserRoleManager(base.BaseManager):
    resource_class = Role

    def list(self, **kwargs):
        """List roles for a given user"""
        # TODO(adriant): Look up user by name/id
        url = '/openstack/users/%s/roles' % kwargs['user']
        return self._list(url, 'roles')

    def add(self, user, role=None, roles=None):
        """Add a role to a user"""
        # TODO(adriant): resolve the roles and users into id's
        # user_id = base.getid(user)
        user_id = user
        # role_id = role
        if role:
            params = {
                'roles': [role]
            }
        elif roles:
            params = {
                'roles': roles
            }

        route = '/openstack/users/%s/roles'
        url = route % (user_id)
        try:
            self._put(url, json=params, response_key=None)
        except exc.HTTPBadRequest as e:
            print(e.message)
            return False

        return True

    def remove(self, user_id, role=None, roles=None):
        """Remove a role or roles from a user"""
        if role:
            params = {
                'roles': [role]
            }
        elif roles:
            params = {
                'roles': roles
            }

        route = '/openstack/users/%s/roles'
        url = route % (user_id)
        try:
            self._delete(url, json=params, response_key=None)
        except exc.HTTPBadRequest as e:
            print(e.message)
            return False

        return True
